function [a,z]=gammagainupdate(yepsilon,a,T,z,p,priorA,priorB)

%Update gains A and auxiliary variables Z under the Poisson observation model
%and Gamma chain prior
%
%Usage: [a,z]=gammagainupdate(yepsilon,a,T,z,p,priorA,priorB)
%
%Input:
%
%ypsilon   observation matrix
%a         gain matrix
%T         template matrix
%z         auxiliary variable matrix
%p         coupling between gains and auxiliry variables
%priorA (optional) alpha parameter of the Gamma prior of gains
%priorB (optional) beta parameter of the Gamma prior of gains
%
%Output:
%
%a         updated gains
%z         updated auxiliry variables
%
%See T. Virtanen, A. T. Cemgil, and S. J. Godsill. "Bayesian Extensions
%to Non-negative Matrix Factorisation for Audio Signal Modelling,"
%ICASSP 2008 for explanation of the terms.

%Tuomas Virtanen 2007


if p~=0
%    ez=(2*p-1)/2;
ez=p;
else
    ez=0;
end

if ~exist('priorA','var')
    priorA=1;
    priorB=0;
end

model=T*a;

notvalid=(yepsilon==0);
espsilon(notvalid)=1;
model(notvalid)=1;


%update amplitudes
neggrads=T'*(yepsilon./model);
posgrads=T'*ones(size(yepsilon));

if p>0
    couplingneggrad=2*ez./a;
    neggrads=neggrads+couplingneggrad+(priorA-1)./a;
    posgrads=posgrads+p*z+p*z(:,[end, 1:end-1]);
end
posgrads=posgrads+priorB;
notvalid=(~posgrads|~neggrads);
posgrads(notvalid)=1;
neggrads(notvalid)=1;
a=a.*neggrads./posgrads;
%set values which are almost zero to zero
a(a<1e-16)=0;

%update coupling parameters
if p>0
%    neggrads=2*ez./(p*z);
%    posgrads=a+a(:,[2:end, 1]);
%    z=z.*neggrads./posgrads;
    neggrads=2*ez/p;
    posgrads=a+a(:,[2:end, 1]);
    z=neggrads./posgrads;
end

