function gA=sparsegrad(A,sparse_c)

%calculates the gradient for the sparseness term
%
%input:    A         gain matrix
%          sparse_c  for of the sparseness function. If scalar,
%                    sparsefun=abs(x). If vector, sparsefun=log(x^2+1)
%                    if sparse_c(2)==1, use sparsefun(x/norm(all_x))
%                    if sparse_c(2)==2, use sparsefun(x/std(x))
%
%output:   gA        gradient of the gain matrix
%

%Tuomas Virtanen 2004-2005


T=size(A,1);

if length(sparse_c)==1
  %sparsefun=abs(x)
  gA=repmat(sum(A.^2,1).^(-1/2),T,1)-repmat(sum(A,1),T,1).*repmat(sum(A.^2,1).^(-3/2),T,1).*A;
else
  %sparsefun=log(x^2+1)
  if sparse_c(2)==1
    gA=2*A./(A.^2+1)-2*repmat(sum((A.^2)./(A.^2+1)),T,1).*A;
  else
    gA=2*T*A./(T*A.^2+1)-2*T*repmat(sum((A.^2)./(T*A.^2+1)),T,1).*A;
  end
end
