function V=variationalupdate(V,E,X)

%Variational update of template matrix V, assuming Gaussian observation model
%for the observation matrix X where the variance is modeled as V * E
%
%Usage: V=variationalupdate(V,E,X)
%
%Input:
%
%V     template matrix
%E     excitation matrix
%X     observation matrix
%
%Output:
%
%V     update template matrix

%Tuomas Virtanen 2007


%warning off MATLAB:divideByZero

sum_lambda = V*E;
sum_lambda(~sum_lambda)=1;

for i=1:size(V,2),
   lambda = V(:,i)*E(i,:);
   kappa = lambda./sum_lambda;
   kappa2= X.*lambda./sum_lambda.^2;

   %S2 = kappa.^2.*X + lambda.*(1 - kappa);
   %V(:,i) = S2*(1./E(i,:))'/size(E,2);
   
   %V2(:,i)=sum((V2(:,i).^2*E(i,:)).*X./(V2*E).^2+V2(:,i)*ones(1,size(E,2))-(V2(:,i).^2*E(i,:))./(V2*E),2)/size(E,2);
   V(:,i)=V(:,i).*(mean(kappa2-kappa,2)+1);
end

%warning on MATLAB:divideByZero
%V(isnan(V))=0;

%set values which are almost zero to zero
V(V<1e-16)=0;

